#! /etc/gosu/0.8/bin/gosu.sh
classpath "../src"

uses gw.lang.cli.*

uses java.util.Calendar
uses java.util.logging.Logger
uses java.util.logging.Level

uses net.thegreshams.gosuZfsUtility.*
uses net.thegreshams.gosuZfsUtility.impl.*
uses net.thegreshams.gosuZfsUtility.Args



/* SET UP THE LOGGER */
final var LOGGER = Logger.getLogger( "net.thegreshams.gosuZfsUtility.gosuZfsManagerRunner" )
final var LOGGER_ENTRY = "ENTRY"
final var LOGGER_EXIT = "EXIT"
LOGGER.setLevel( Level.ALL )
LOGGER.info( LOGGER_ENTRY )



/* CONFIGURE THE COMMAND-LINE PARSER */
LOGGER.info( "Initializing command-line parser." )
CommandLineAccess.initialize( Args )



/* INITIALIZE OPERATIONAL-MODE (ie: testing or not) */
if( Args.TestMode ) {
    
    LOGGER.info( "Test-mode is ENABLED." )
    LOGGER.setLevel( Level.ALL )
    
} else {
    
    LOGGER.info( "Test-mode is DISABLED." )
    LOGGER.setLevel( Level.WARNING )
    
}



/* EXECUTE */
var manager : net.thegreshams.gosuZfsUtility.GosuZfsManager
var shell : net.thegreshams.gosuZfsUtility.GosuZfsShell
if( Args.TestMode ) {
    
	shell = new GosuZfsEmulator(
					"resources/zfsEmulatorResults/listAll",
					Args.UseKeywordSudo,
					Args.VolumeName, 
					Args.LineDelimeter, 
					Args.VolumeNameSnapshotTypeSeparator )
					
	manager = GosuZfsManager.newInstanceRunningInTestMode( shell )	

	configureTheManager()
	
	exerciseTheEmulator();
	exerciseTheManager();
				
} else {
      
    shell = new net.thegreshams.gosuZfsUtility.impl.GosuZfsShellImpl()
    
    manager = GosuZfsManager.newInstance( shell )
    
    configureTheManager()
    
	manager.performMaintenance()
    
}


/* EXIT */
LOGGER.setLevel( Level.ALL )
LOGGER.info( LOGGER_EXIT )
return 0



/********************************************************************************************************************
 * 
 * PRIVATE CONVENIENCE-METHODS
 * 
 *******************************************************************************************************************/



		/* CONFIGURE THE MANAGER */
		private function configureTheManager() {
		    
		    manager.ZfsVolume = Args.VolumeName
			manager.ZfsVolumeSnapshotTypeSeparator = Args.VolumeNameSnapshotTypeSeparator
			manager.ZfsSnapshotTypeDateSeparator = Args.SnapshotTypeDateSeparator
			manager.ZfsDelimeter = Args.LineDelimeter
			manager.ArchiveDaysToMaintain = Args.ArchiveDaysToMaintain
			manager.ArchiveWeeksToMaintain = Args.ArchiveWeeksToMaintain
			manager.ArchiveWeeklyDay = Args.ArchiveDayOfWeeklyMaintenance
			manager.ArchiveMonthsToMaintain = Args.ArchiveMonthsToMaintain
			manager.ArchiveMonthlyDate = Args.ArchiveDayOfMonthlyMaintenance
			manager.ArchiveYearsToMaintain = Args.ArchiveYearsToMaintain
			manager.ArchiveYearlyMonth = Args.ArchiveMonthOfAnnualMaintenance
			manager.ArchiveYearlyDate = Args.ArchiveDayOfAnnualMaintenance
			manager.ArchiveDailyTag = Args.DayTag
			manager.ArchiveWeeklyTag = Args.WeekTag
			manager.ArchiveMonthlyTag = Args.MonthTag
			manager.ArchiveYearlyTag = Args.YearTag
			manager.Keyword_sudo = Args.KeywordSudo
			manager.Use_Keyword_Sudo = Args.UseKeywordSudo
			manager.Keyword_zfs = Args.KeywordZfs
			manager.Keyword_snapshot = Args.KeywordZfsSnapshot
			manager.Keyword_destroy = Args.KeywordZfsDestroy
			manager.Keyword_listAll = Args.KeywordZfsListAll			
		}


		/* TEST THE EMULATOR */
		private function exerciseTheEmulator() {
			   
			LOGGER.info( LOGGER_ENTRY )
		
			var cmdToExecute = "${Args.KeywordZfs} ${Args.KeywordZfsListAll}"
			var result = shell.execute( cmdToExecute )
			LOGGER.info( result )
			
			cmdToExecute = "${Args.KeywordZfs} ${Args.KeywordZfsDestroy} tank@day_20101005"
			result = shell.execute( cmdToExecute )
			LOGGER.info( result )
			
			// run again to test inability to destroy what's already gone
			result = shell.execute( cmdToExecute )
			LOGGER.info( result )
			
			cmdToExecute = "${Args.KeywordZfs} ${Args.KeywordZfsSnapshot} tank@day_20101209"
			result = shell.execute( cmdToExecute )
			LOGGER.info( result )
			
			// run again to test inability to create what already exists
			result = shell.execute( cmdToExecute )
			LOGGER.info( result )
			
			LOGGER.info( LOGGER_EXIT )
		}
		
		
		
		/* TEST THE MANAGER */
		private function exerciseTheManager() {
		 
		    LOGGER.info( LOGGER_ENTRY )
		    
			
			var listAllCommand = "${manager.Keyword_zfs} ${manager.Keyword_listAll}"
			if( Args.UseKeywordSudo ) {
			    listAllCommand = "${manager.Keyword_sudo} ${listAllCommand}"
			}
			
			LOGGER.info( shell.execute( listAllCommand ) )
			
			manager.performMaintenance()
			
			/* Should perform a daily */
			manager.TestParameter_Year = 2009
			manager.TestParameter_Month = 1
			manager.TestParameter_Day = 12
			manager.TestParameter_DayOfTheWeek = Calendar.SUNDAY
			manager.performMaintenance()
			
			/* Should perform a weekly */
			manager.TestParameter_Day = 17
			manager.TestParameter_DayOfTheWeek = Calendar.FRIDAY
			manager.performMaintenance()
			
			/* Should perform a weekly */
			manager.TestParameter_Day = 24
			manager.performMaintenance()
			
			/* Should perform a weekly */
			manager.TestParameter_Day = 31
			manager.performMaintenance()
			
			/* Should perform a monthly */
			manager.TestParameter_Day = 1
			manager.TestParameter_DayOfTheWeek = Calendar.SATURDAY
			manager.TestParameter_Month = 1
			manager.TestParameter_Year = 2010
			manager.performMaintenance()
			
			/* Should perform a monthly */			
			manager.TestParameter_DayOfTheWeek = Calendar.TUESDAY
			manager.TestParameter_Month = 2
			manager.performMaintenance()
			
			/* Should perform a monthly */			
			manager.TestParameter_DayOfTheWeek = Calendar.TUESDAY
			manager.TestParameter_Month = 3
			manager.performMaintenance()
			
			/* Should perform a monthly */			
			manager.TestParameter_DayOfTheWeek = Calendar.FRIDAY
			manager.TestParameter_Month = 4
			manager.performMaintenance()
			
			/* Should perform a monthly */			
			manager.TestParameter_DayOfTheWeek = Calendar.SATURDAY
			manager.TestParameter_Month = 5
			manager.performMaintenance()
			
			/* Should perform a monthly */			
			manager.TestParameter_DayOfTheWeek = Calendar.WEDNESDAY
			manager.TestParameter_Month = 6
			manager.performMaintenance()
			
			/* Should perform a monthly */			
			manager.TestParameter_DayOfTheWeek = Calendar.FRIDAY
			manager.TestParameter_Month = 7
			manager.performMaintenance()
			
			/* Should perform a monthly */			
			manager.TestParameter_DayOfTheWeek = Calendar.MONDAY
			manager.TestParameter_Month = 8
			manager.performMaintenance()
			
			/* Should perform a monthly */			
			manager.TestParameter_DayOfTheWeek = Calendar.THURSDAY
			manager.TestParameter_Month = 9
			manager.performMaintenance()
			
			/* Should perform a monthly */			
			manager.TestParameter_DayOfTheWeek = Calendar.SATURDAY
			manager.TestParameter_Month = 10
			manager.performMaintenance()
			
			/* Should perform a monthly */			
			manager.TestParameter_DayOfTheWeek = Calendar.TUESDAY
			manager.TestParameter_Month = 11
			manager.performMaintenance()
			
			/* Should perform a monthly */			
			manager.TestParameter_DayOfTheWeek = Calendar.THURSDAY
			manager.TestParameter_Month = 12
			manager.performMaintenance()
			
			/* Should perform an annual */
			manager.TestParameter_DayOfTheWeek = Calendar.SUNDAY
			manager.TestParameter_Day = 31
			manager.performMaintenance()
			
			/* Should perform a monthly */
			manager.TestParameter_Day = 1
			manager.TestParameter_Month = 1
			manager.TestParameter_Year = 2011
			manager.performMaintenance()
			
			/* Should perform a daily */
			manager.TestParameter_Day = 2
			manager.performMaintenance()
			
			LOGGER.info( shell.execute( listAllCommand ) )
		
			LOGGER.info( LOGGER_EXIT )
		}
		
