package net.thegreshams.gosuZfsUtility

uses java.util.Arrays
uses java.util.Calendar
uses java.util.logging.Level
uses java.util.logging.Logger

class GosuZfsManager {
    
    
	final var _isTestModeEnabled : boolean
	final var _zShell : GosuZfsShell
	
	final static var LOGGER = Logger.getLogger( GosuZfsManager.Type.Name )
	final static var LOGGER_TESTING_MODE_ENABLED = "TESTING MODE IS ENABLED."
	final static var LOGGER_STANDARD_MODE_ENABLED = "RUNNING IN STANDARD MODE."
	final static var LOGGER_ENTRY = "ENTRY"
	final static var LOGGER_EXIT = "EXIT"
	final static var LOGGER_ANNUAL_SNAPSHOT_TAKEN = "Annual snapshot taken:"
	final static var LOGGER_MONTHLY_SNAPSHOT_TAKEN = "Monthly snapshot taken:"	
	final static var LOGGER_WEEKLY_SNAPSHOT_TAKEN = "Weekly snapshot taken:"	
	final static var LOGGER_DAILY_SNAPSHOT_TAKEN = "Daily snapshot taken:"	
	final static var LOGGER_SNAPSHOT_ALREADY_TAKEN = "Prior snapshot has been taken already -- returning early."
	final static var LOGGER_DATE_SEPARATOR = "/"
	final static var LOGGER_DAY_IS_NOT_TARGET_FOR_MONTHLY_SNAPSHOTS_1 =
							 	"Current day is not the day targeted for monthly-snapshots. Current day is '"
	final static var LOGGER_DAY_IS_NOT_TARGET_FOR_MONTHLY_SNAPSHOTS_2 =
								"' and archive target-day is '"
	final static var LOGGER_DAY_IS_NOT_TARGET_FOR_MONTHLY_SNAPSHOTS_3 =
								"'. Returning early."
	final static var LOGGER_DAY_IS_NOT_TARGET_FOR_ANNUAL_SNAPSHOTS_1 = 
								"Current month/day are not a target for annual-snapshots. Current month/day are '"
	final static var LOGGER_DAY_IS_NOT_TARGET_FOR_ANNUAL_SNAPSHOTS_2 = 
								"' and archive target-month/day are '"
	final static var LOGGER_DAY_IS_NOT_TARGET_FOR_ANNUAL_SNAPSHOTS_3 =
								"'. Returning early."
	final static var LOGGER_DAY_IS_NOT_TARGET_FOR_WEEKLY_SNAPSHOTS_1 =
								"Current doy-of-the-week is not a target for weekly-snapshots. Current day-of-the-week is '"
	final static var LOGGER_DAY_IS_NOT_TARGET_FOR_WEEKLY_SNAPSHOTS_2 =
								"' and archive target-day-of-the-week is '"
	final static var LOGGER_DAY_IS_NOT_TARGET_FOR_WEEKLY_SNAPSHOTS_3 =
								"'. Returning early."
	final static var LOGGER_RECEIVED_THE_FOLLOWING_SNAPSHOTS_OF_TYPE =
								"Received the following snapshots of type"
	final static var LOGGER_THE_UNTRIMMED_LIST_CONTAINS_N_ENTRIES_1 =
								"The untrimmed list contains"
	final static var LOGGER_THE_UNTRIMMED_LIST_CONTAINS_N_ENTRIES_2 =
								"entries."
	final static var LOGGER_NEED_TO_KEEP_N_ENTRIES_1 =
								"Need to keep"
	final static var LOGGER_NEED_TO_KEEP_N_ENTRIES_2 =
								"entries."
	final static var LOGGER_TRIMMING_N_ENTRIES_1 =
								"Trimming"
	final static var LOGGER_TRIMMING_N_ENTRIES_2 =
								"entries."
	final static var LOGGER_RETURNING_A_LIST_FROM_INDEX_TO_INDEX_1 =
								"Returning a list from index"
	final static var LOGGER_RETURNING_A_LIST_FROM_INDEX_TO_INDEX_2 =
								"to index"
								
	var _zfsVolume : String as ZfsVolume
	var _zfsVolumeSnapshotTypeSeparator : String as ZfsVolumeSnapshotTypeSeparator
	var _zfsSnapshotTypeDateSeparator : String as ZfsSnapshotTypeDateSeparator
	var _zfsDelimeter : String as ZfsDelimeter
	var _archiveDaysToMaintain : int as ArchiveDaysToMaintain
	var _archiveWeeksToMaintain : int as ArchiveWeeksToMaintain
	var _archiveWeeklyDay : int
		property get ArchiveWeeklyDay() : String {
		    return Calendar.DayOfWeekFromInt( this._archiveWeeklyDay )
		}
		property set ArchiveWeeklyDay( dayOfWeek : String ) {
		    this._archiveWeeklyDay = Calendar.DayOfWeekFromString( dayOfWeek )
		}
	var _archiveMonthsToMaintain : int as ArchiveMonthsToMaintain
	var _archiveMonthlyDate : int as ArchiveMonthlyDate
	var _archiveYearsToMaintain : int as ArchiveYearsToMaintain
	var _archiveYearlyDate : int as ArchiveYearlyDate
	var _archiveYearlyMonth : int as ArchiveYearlyMonth
	var _archiveTagDay : String as ArchiveDailyTag
	var _archiveTagWeek : String as ArchiveWeeklyTag
	var _archiveTagMonth : String as ArchiveMonthlyTag
	var _archiveTagYear : String as ArchiveYearlyTag
	var _use_keyword_sudo : boolean as Use_Keyword_Sudo
	var _keyword_sudo : String as Keyword_sudo
	var _keyword_zfs : String as Keyword_zfs
	var _keyword_listAll : String as Keyword_listAll
	var _keyword_snapshot : String as Keyword_snapshot
	var _keyword_destroy : String as Keyword_destroy
	var _testParameter_dayOfWeek : int as TestParameter_DayOfTheWeek
	var _testParameter_day : int as TestParameter_Day
	var _testParameter_month : int as TestParameter_Month
	var _testParameter_year : int as TestParameter_Year
	
	
	
	private construct( isTestModeEnabled : boolean, zShell : GosuZfsShell )  { 	    
	    
	    LOGGER.setLevel( Level.ALL )
	    LOGGER.info( LOGGER_ENTRY )
	    
	    this._isTestModeEnabled = isTestModeEnabled
	    if( this._isTestModeEnabled ) {
	        LOGGER.setLevel( Level.INFO )
			LOGGER.info( LOGGER_TESTING_MODE_ENABLED )
	    } else {
	        LOGGER.info( LOGGER_STANDARD_MODE_ENABLED )
	        LOGGER.setLevel( Level.SEVERE )
	    }
	    
	    if( zShell != null ) {
	        this._zShell = zShell
	    }    
	    
	    LOGGER.info( LOGGER_EXIT )
	}

	static function newInstance( zShell : GosuZfsShell ) : GosuZfsManager {
	   
	    var result : GosuZfsManager	    
	    LOGGER.info( LOGGER_ENTRY )
	    
	    result = new GosuZfsManager( false, zShell )
	    
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	static function newInstanceRunningInTestMode( zShell : GosuZfsShell ) : GosuZfsManager {
	    
	    var result : GosuZfsManager    
	    LOGGER.info( LOGGER_ENTRY )
	    
	    result = new GosuZfsManager( true, zShell )	
	    
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	function performMaintenance() {
	    
	    var alreadyPerformedSnapshotForToday = false
	    LOGGER.info( LOGGER_ENTRY )	    
	    	    	    
	    logInitializedParameters()	    
	    
	    /* ANNUAL */
	    alreadyPerformedSnapshotForToday = performAnnualMaintenance( alreadyPerformedSnapshotForToday )
	    LOGGER.info( LOGGER_ANNUAL_SNAPSHOT_TAKEN + alreadyPerformedSnapshotForToday )
	  	    
	    
	    /* MONTHLY */
	    alreadyPerformedSnapshotForToday = performMonthlyMaintenance( alreadyPerformedSnapshotForToday )
	    LOGGER.info( LOGGER_MONTHLY_SNAPSHOT_TAKEN + alreadyPerformedSnapshotForToday )	    
	        
	    
	    /* WEEKLY */
		alreadyPerformedSnapshotForToday = performWeeklyMaintenance( alreadyPerformedSnapshotForToday )
	    LOGGER.info( LOGGER_WEEKLY_SNAPSHOT_TAKEN + alreadyPerformedSnapshotForToday )		
	        
	    
	    /* DAILY */
	    alreadyPerformedSnapshotForToday = performDailyMaintenance( alreadyPerformedSnapshotForToday )
	    LOGGER.info( LOGGER_DAILY_SNAPSHOT_TAKEN + alreadyPerformedSnapshotForToday )	    
    
	    
	    LOGGER.info( LOGGER_EXIT )
	}
	
	private function performAnnualMaintenance( hasSnapshotAlreadyBeenTakenToday : boolean ) : boolean {
	    
	    var result = hasSnapshotAlreadyBeenTakenToday
	    LOGGER.info( LOGGER_ENTRY )
	    
	    // if a prior process has already taken a snapshot today, this script shouldn't run
	    if( result ) {
	        LOGGER.info( LOGGER_SNAPSHOT_ALREADY_TAKEN )
	        LOGGER.info( LOGGER_EXIT )
	        return result
	    }
	    
	    var cal = Calendar.getInstance()
	 	var month = getCurrentMonth()
	 	var day = getCurrentDay()
	 	
	 	
	 	/* If today is not the month and day targeted to do an annual-snapshot,
	 	 * return early.
	 	 */
	 	if( month != this._archiveYearlyMonth
	 		or day != this._archiveYearlyDate )
 		{ 		    
 		    LOGGER.info( "${LOGGER_DAY_IS_NOT_TARGET_FOR_ANNUAL_SNAPSHOTS_1} ${month} ${LOGGER_DATE_SEPARATOR} ${day} " +
	 	    			 "${LOGGER_DAY_IS_NOT_TARGET_FOR_ANNUAL_SNAPSHOTS_2} ${this._archiveYearlyMonth} ${LOGGER_DATE_SEPARATOR} ${this._archiveYearlyDate} " +
	 	    			 "${LOGGER_DAY_IS_NOT_TARGET_FOR_ANNUAL_SNAPSHOTS_3}" )	 	    
 		    return result
 		}
	    
	    	
    	// perform today's snapshot as the yearly one
    	var currentDate = getCurrentDate()
    	var cmdToExecute = 
    			"${getCommandSudoPrefix()}${this._keyword_zfs} ${this._keyword_snapshot} " +
    			"${this._zfsVolume}${this._zfsVolumeSnapshotTypeSeparator}${this._archiveTagYear}${this._zfsSnapshotTypeDateSeparator}${currentDate}"
		this._zShell.execute( cmdToExecute )
		result = true
		  
		// remove all snapshots older than previous n-years       
	    var annualSnapshots = getAllSnapshotsOfTypeOlderThanN( this._archiveTagYear, this._archiveYearsToMaintain )
	    for( ss in annualSnapshots ) {
	        cmdToExecute = "${getCommandSudoPrefix()}${this._keyword_zfs} ${this._keyword_destroy} ${ss}"
	        this._zShell.execute( cmdToExecute )
	    }
	    
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	private function performMonthlyMaintenance( hasSnapshotAlreadyBeenTakenToday : boolean ) : boolean {
	    
	    var result = hasSnapshotAlreadyBeenTakenToday
	    LOGGER.info( LOGGER_ENTRY )
	    
	    // if a prior process has already taken a snapshot today, this script shouldn't run
	    if( result ) {
	        LOGGER.info( LOGGER_SNAPSHOT_ALREADY_TAKEN )
	        LOGGER.info( LOGGER_EXIT )
	        return result
	    }
	    
	 	var day = getCurrentDay()	 	
	 	
	 	/* If today is not the day targeted to do a monthly-snapshot,
	 	 * return early.
	 	 */
	 	if( day != this._archiveMonthlyDate ) {
	 	    LOGGER.info( "${LOGGER_DAY_IS_NOT_TARGET_FOR_MONTHLY_SNAPSHOTS_1} ${day} " +
	 	    			 "${LOGGER_DAY_IS_NOT_TARGET_FOR_MONTHLY_SNAPSHOTS_2} ${this._archiveMonthlyDate} " +
	 	    			 "${LOGGER_DAY_IS_NOT_TARGET_FOR_MONTHLY_SNAPSHOTS_3}" )
	 	    return result
	 	}
	    
	    // perform today's snapshot as the monthly one
    	var currentDate = getCurrentDate()
    	var cmdToExecute =
    			"${getCommandSudoPrefix()}${this._keyword_zfs} ${this._keyword_snapshot} " +
    			"${this._zfsVolume}${this._zfsVolumeSnapshotTypeSeparator}${this._archiveTagMonth}${this._zfsSnapshotTypeDateSeparator}${currentDate}"    			
		this._zShell.execute( cmdToExecute )
		result = true
		  
		// remove all snapshots older than previous n-months       
	    var monthlySnapshots = getAllSnapshotsOfTypeOlderThanN( this._archiveTagMonth, this._archiveMonthsToMaintain )
	    for( ss in monthlySnapshots ) {
	        cmdToExecute = "${getCommandSudoPrefix()}${this._keyword_zfs} ${this._keyword_destroy} ${ss}"
	        this._zShell.execute( cmdToExecute )
	    }
	    
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	private function performWeeklyMaintenance( hasSnapshotAlreadyBeenTakenToday : boolean ) : boolean {
	    
	    var result = hasSnapshotAlreadyBeenTakenToday
	    LOGGER.info( LOGGER_ENTRY )
	    
	    // if a prior process has already taken a snapshot today, this script shouldn't run
	    if( result ) {
	        LOGGER.info( LOGGER_SNAPSHOT_ALREADY_TAKEN )
	        LOGGER.info( LOGGER_EXIT )	        
	        return result
	    }
	    
	    // get the current day of the week
	    var dow = getCurrentDayOfWeek()
	    
	    // if today is not the day to perform weekly-maintenance, this script shouldn't run
	    if( dow != this._archiveWeeklyDay ) {
	        LOGGER.info( "${LOGGER_DAY_IS_NOT_TARGET_FOR_ANNUAL_SNAPSHOTS_1} ${dow} " +
	        			 "${LOGGER_DAY_IS_NOT_TARGET_FOR_ANNUAL_SNAPSHOTS_2} ${this._archiveWeeklyDay} ${this._archiveWeeklyDay} " +
	        			 "${LOGGER_DAY_IS_NOT_TARGET_FOR_ANNUAL_SNAPSHOTS_3}" )
	    	return result
	    }
	    
	    // perform today's snapshot as the weekly one
    	var currentDate = getCurrentDate()
	    var cmdToExecute = 
	    		"${getCommandSudoPrefix()}${this._keyword_zfs} ${this._keyword_snapshot} " +
	    		"${this._zfsVolume}${this._zfsVolumeSnapshotTypeSeparator}${this._archiveTagWeek}${this._zfsVolumeSnapshotTypeSeparator}${currentDate}"
		this._zShell.execute( cmdToExecute )
		result = true
		
		// remove all snapshots older than previous n-weeks
		var weeklySnapshots = getAllSnapshotsOfTypeOlderThanN( this._archiveTagWeek, this._archiveWeeksToMaintain )
		for( ss in weeklySnapshots ) {
		    cmdToExecute = "${getCommandSudoPrefix()}${this._keyword_zfs} ${this._keyword_destroy} ${ss}"
		    this._zShell.execute( cmdToExecute )
		}
	    
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	private function performDailyMaintenance( hasSnapshotAlreadyBeenTakenToday : boolean ) : boolean {	  
  
	    var result = hasSnapshotAlreadyBeenTakenToday
	    LOGGER.info( LOGGER_ENTRY )
	    
	    // if a prior process has already taken a snapshot today, this script shouldn't run
	    if( hasSnapshotAlreadyBeenTakenToday ) {
	        LOGGER.info( LOGGER_SNAPSHOT_ALREADY_TAKEN )
	        LOGGER.info( LOGGER_EXIT )
	        return result
	    }
	    
	    // perform today's snapshot
	    var currentDate = getCurrentDate()
	    var cmdToExecute =
	    		"${getCommandSudoPrefix()}${this._keyword_zfs} ${this._keyword_snapshot} " +
	    		"${this._zfsVolume}${this._zfsVolumeSnapshotTypeSeparator}${this._archiveTagDay}${this._zfsSnapshotTypeDateSeparator}${currentDate}"   		
	    this._zShell.execute( cmdToExecute )    
	    result = true
	    
	    
	    // remove all snapshots older than previous n-days	       
	    var dailySnapshots = getAllSnapshotsOfTypeOlderThanN( this._archiveTagDay, this._archiveDaysToMaintain )
	    for( ss in dailySnapshots ) {
	        cmdToExecute = "${getCommandSudoPrefix()}${this._keyword_zfs} ${this._keyword_destroy} ${ss}"
	        this._zShell.execute( cmdToExecute )
	    }
	    
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	private function getCurrentDate() : String {
   
		var result = ""
		LOGGER.info( LOGGER_ENTRY )
	 
	 	var cal = Calendar.getInstance()
	 	var day = "${ cal.CalendarDay }"
	 	var month = "${ cal.CalendarMonth + 1 }"
	 	var year = "${ cal.CalendarYear }"
	 	
	 	if( this._isTestModeEnabled ) {
	 	    
	 	    if( this._testParameter_day >= 1 and this._testParameter_day <= 31 ) {
	 	        day = "${this._testParameter_day}"
	 	    }
	 	    
	 	    if( this._testParameter_month >= 1 and this._testParameter_month <= 12 ) {
	 	        month = "${this._testParameter_month}"
	 	    }
	 	    
	 	    if( this._testParameter_year >= 2000 ) {
	 	        year = "${this._testParameter_year}"
	 	    }
	 	}
	 
	 	if( month.length() < 2 ) {
	 	   month = "0${month}" 
	 	}
	 	if( day.length() < 2 ) {
	 	   day = "0${day}" 
	 	}
	 
	 	result = "${year}${month}${day}"
	 
	 	LOGGER.info( LOGGER_EXIT ) 
 		return result 
	}
	
	private function getCurrentYear() : int {
	    
	    var result = Calendar.getInstance().CalendarYear
	    LOGGER.info( LOGGER_ENTRY )
	    
	    if( this._isTestModeEnabled ) {
	        
	        if( this._testParameter_year >= 2000 ) {
	            result = this._testParameter_year
	        }
	    }
	    
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	private function getCurrentMonth() : int {
	    
	    var result = Calendar.getInstance().CalendarMonth + 1
	    LOGGER.info( LOGGER_ENTRY )
	    
	    if( this._isTestModeEnabled ) {
	        
	        if( this._testParameter_month >= 1 and this._testParameter_month <= 12 ) {
	            result = this._testParameter_month
	        }
	    }
	    
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	private function getCurrentDayOfWeek() : int {
	   
	    var result = Calendar.getInstance().CalendarDayOfWeek
	    LOGGER.info( LOGGER_ENTRY )
	    
	    if( this._isTestModeEnabled ) {
	    
	        if( this._testParameter_dayOfWeek >= Calendar.SUNDAY and this._testParameter_dayOfWeek <= Calendar.SATURDAY ) {
	            result = this._testParameter_dayOfWeek
	        }
	        
	    }
	    
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	private function getCurrentDay() : int {
	    
	    var result = Calendar.getInstance().CalendarDay
	    LOGGER.info( LOGGER_ENTRY )
	    
	    if( this._isTestModeEnabled ) {
	        
	        if( this._testParameter_day >= 1 and this._testParameter_day <= 31 ) {
	            result = this._testParameter_day
	        }
	    }
	    
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	private function getAllSnapshotsOfTypeOlderThanN( typeByDayWeekMonthOrYear : String, numberOfNewestSnapshotsToKeep : int ) : List<String> {
	    
	    var result : List<String> = {}
	    LOGGER.info( LOGGER_EXIT )
	    
	    var allSnapshotsOfType = getAllSnapshotsOfType( typeByDayWeekMonthOrYear )
	    var message = "${LOGGER_RECEIVED_THE_FOLLOWING_SNAPSHOTS_OF_TYPE} ${typeByDayWeekMonthOrYear}: "
	    for( var snapshot in allSnapshotsOfType ) {
	        message += "${snapshot} "
	    }
	    LOGGER.info( message )
	    
	    var startIndex = numberOfNewestSnapshotsToKeep
	    if( numberOfNewestSnapshotsToKeep > allSnapshotsOfType.Count ) {
	        startIndex = allSnapshotsOfType.Count
	    }
	    
	    message = "${LOGGER_THE_UNTRIMMED_LIST_CONTAINS_N_ENTRIES_1} ${allSnapshotsOfType.Count} ${LOGGER_THE_UNTRIMMED_LIST_CONTAINS_N_ENTRIES_2}"
	    LOGGER.info( message )
	    
	    message = "${LOGGER_NEED_TO_KEEP_N_ENTRIES_1} ${numberOfNewestSnapshotsToKeep} ${LOGGER_NEED_TO_KEEP_N_ENTRIES_2}"
	    LOGGER.info( message )		
	    
	    message = "${LOGGER_TRIMMING_N_ENTRIES_1} ${allSnapshotsOfType.Count - startIndex} ${LOGGER_TRIMMING_N_ENTRIES_2}"
	    LOGGER.info( message )
	    
	    message = "${LOGGER_RETURNING_A_LIST_FROM_INDEX_TO_INDEX_1} ${startIndex} ${LOGGER_RETURNING_A_LIST_FROM_INDEX_TO_INDEX_2} ${allSnapshotsOfType.Count}"
	    LOGGER.info( message )
	    
	    result.addAll( allSnapshotsOfType.subList( startIndex, allSnapshotsOfType.Count ) )
	    	    
	   	LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	private function getAllSnapshotsOfType( typeByDayWeekMonthOrYear : String ) : List<String> {
	    
	    var result : List<String> = {}
	    LOGGER.info( LOGGER_ENTRY )
	    
	    var targetStringType = this._archiveTagDay
	    switch( typeByDayWeekMonthOrYear.trim() ) {
	    	case this._archiveTagDay:
	    		targetStringType = this._archiveTagDay
	    		break;
    		case this._archiveTagWeek:
    			targetStringType = this._archiveTagWeek
    			break;
			case this._archiveTagMonth:
				targetStringType = this._archiveTagMonth
				break;
			case this._archiveTagYear:
				targetStringType = this._archiveTagYear
				break;
			default:
				targetStringType = this._archiveTagYear
				break;
	    }
	    
	    
	    var cmdToExecute = "${getCommandSudoPrefix()}${this._keyword_zfs} ${this._keyword_listAll}"
	    var shellResult = this._zShell.execute( cmdToExecute )
	    
	    var targetLine = "${this._zfsVolume}${this._zfsVolumeSnapshotTypeSeparator}${targetStringType}${this._zfsSnapshotTypeDateSeparator}"
	    for( line in shellResult.split( this._zfsDelimeter ) ) {

	        if( line.trim().contains( targetLine ) ) {	
	                     
	            result.add( line.trim().split( " " )[0] )	            
	            
	        }
	        
	    }
	    
	    result.sortDescending()
	    
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	private function logInitializedParameters() {
	    
	    LOGGER.info( LOGGER_ENTRY )
	    
	    logInitializedParameter( "_zfsVolume", _zfsVolume )
	    logInitializedParameter( "_zfsVolumeSnapshotTypeSeparator", _zfsVolumeSnapshotTypeSeparator )	    
	    logInitializedParameter( "_zfsSnapshotTypeDateSeparator", _zfsSnapshotTypeDateSeparator )	    
	    logInitializedParameter( "_zfsDelimeter", _zfsDelimeter )	    
	    logInitializedParameter( "_archiveDaysToMaintain", _archiveDaysToMaintain as String )	    
	    logInitializedParameter( "_archiveWeeksToMaintain", _archiveWeeksToMaintain as String )	      
	    logInitializedParameter( "_archiveWeeklyDay", _archiveWeeklyDay as String )	     
	    logInitializedParameter( "_archiveMonthsToMaintain", _archiveMonthsToMaintain as String )	     
	    logInitializedParameter( "_archiveMonthlyDate", _archiveMonthlyDate as String )	     
	    logInitializedParameter( "_archiveYearsToMaintain", _archiveYearsToMaintain as String )	     
	    logInitializedParameter( "_archiveYearlyDate", _archiveYearlyDate as String )	     
	    logInitializedParameter( "_archiveYearlyMonth", _archiveYearlyMonth as String )	   
	    logInitializedParameter( "_archiveTagDay", _archiveTagDay )	 	    
	    logInitializedParameter( "_archiveTagWeek", _archiveTagWeek )	 	    
	    logInitializedParameter( "_archiveTagMonth", _archiveTagMonth )	 	    
	    logInitializedParameter( "_archiveTagYear", _archiveTagYear )	 	    
	    logInitializedParameter( "_use_keyword_sudo", _use_keyword_sudo as String )	 	    
	    logInitializedParameter( "_keyword_sudo", _keyword_sudo )	 	    
	    logInitializedParameter( "_keyword_zfs", _keyword_zfs )	 	    
	    logInitializedParameter( "_keyword_listAll", _keyword_listAll )	
	    logInitializedParameter( "_keyword_snapshot", _keyword_snapshot )		    
	    logInitializedParameter( "_keyword_destroy", _keyword_destroy )		    
	    logInitializedParameter( "_testParameter_dayOfWeek", _testParameter_dayOfWeek as String )		    
	    logInitializedParameter( "_testParameter_day", _testParameter_day as String )		    
	    logInitializedParameter( "_testParameter_month", _testParameter_month as String )		    
	    logInitializedParameter( "_testParameter_year", _testParameter_year as String )		    	    
	    
	    LOGGER.info( LOGGER_EXIT )
	}
	
	private function logInitializedParameter( paramName : String, paramValue : String ) {
	    
	    if( paramName == null ) {
	    	LOGGER.warning( "Function called with null supplied for paramName." )
	    	return
	    }
	    
	    paramName = paramName.trim()
	    if( paramValue == null ) {
	        paramValue = "<null>"
	    } else {
	        paramValue = paramValue.trim()
	    }
	    
	    var originalLevel = LOGGER.Level
	    LOGGER.setLevel( Level.ALL )
	    LOGGER.info( "${paramName} = ${paramValue}" )
	    LOGGER.setLevel( originalLevel )
	}
	
	private function getCommandSudoPrefix() : String {
	    
	    var result = ""
	    LOGGER.info( LOGGER_ENTRY )
	    
	    if( _use_keyword_sudo ) {
	        result = "${_keyword_sudo} "
	    }
   
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
}
